#!/usr/bin/env python3

import os
import signal
from scapy.all import *
from netfilterqueue import NetfilterQueue
import argparse

# 默认窗口大小，若未传入则使用这些值
initial_window_size = 17  # 初始窗口大小
normal_window_size = 65535  # 正常窗口大小

def modify_window(pkt):
    try:
        raw_pkt = pkt.get_payload()
        ip = IP(raw_pkt) if raw_pkt[0] == 0x45 else IPv6(raw_pkt)

        if ip.haslayer(TCP):
            tcp = ip[TCP]

            # 在握手阶段（SYN 或 SYN-ACK）使用初始窗口大小
            if tcp.flags in ["S", "SA"]:
                tcp.window = initial_window_size
            # 在后续阶段（ACK 或 PSH-ACK）恢复为正常窗口大小
            elif tcp.flags in ["A", "PA"]:
                tcp.window = normal_window_size

            # 删除旧的校验和以便重新计算
            del ip.chksum
            del tcp.chksum
            pkt.set_payload(bytes(ip))
    except Exception as e:
        print(f"Error processing packet: {e}")

    pkt.accept()

def parsearg():
    global initial_window_size  # 将全局变量设置为命令行传入的值
    parser = argparse.ArgumentParser(description='TCP Window Size Modifier')

    parser.add_argument('-q', '--queue', type=int, required=True, help='iptables Queue Num')
    parser.add_argument('-w', '--window_size', type=int, required=True, help='Initial TCP Window Size')

    args = parser.parse_args()

    initial_window_size = args.window_size  # 设置初始窗口大小为传入的值
    return args.queue

def main():
    queue_num = parsearg()
    nfqueue = NetfilterQueue()
    nfqueue.bind(queue_num, modify_window)

    try:
        print(f"Starting netfilter_queue process with initial window size: {initial_window_size}")
        nfqueue.run()
    except KeyboardInterrupt:
        pass

if __name__ == "__main__":
    signal.signal(signal.SIGINT, lambda signal, frame: sys.exit(0))
    main()
